             .title        "PCI556 (iNES 38) Reader Plugin"


;08/27/2010 
;Written by BootGod
;Version 1.0

;Board found in game Crime Busters
;Published by Gradiente / Developed by Bit Corp
;Similar to GNROM, no bus-conflicts, register mapped to $7000-7FFF
;bits 0-1 swap 32K PRG
;bits 2-3 swap 8K CHR
             
             ;vectors for standard system calls

send_byte:   .equ 0200h
baton:       .equ 0203h
chk_vram:    .equ 0206h
chk_wram:    .equ 0209h
wr_ppu:      .equ 020ch
read_byte:   .equ 020fh
init_crc:    .equ 0212h
do_crc:      .equ 0215h
finish_crc:  .equ 0218h

crc0:        .equ 0080h
crc1:        .equ 0081h
crc2:        .equ 0082h
crc3:        .equ 0083h

address:       .equ 00e0h
address_lo:    .equ 00e0h
address_hi:    .equ 00e1h

curbank:    .equ 00e2h


temp_crc:    .equ 00ech

;plugin header defines
p_ver: = 1	;plugin version
p_read: = 1	;plugin is for downloading (dumping)
p_write: = 2	;plugin is for uploading (RAM carts / WRAM etc)
p_other: = 3	;plugin is for some other purpose (i.e. BankWatch)
p_var1: = 4	;plugin uses user variable #1
p_var2: = 8	;plugin uses user variable #2
p_var3: = 16	;plugin uses user variable #3
p_var4: = 32	;plugin uses user variable #4

;plugin header data

             .org 0380h
             
             ;plugin description
             .db "PCI556 carts"
             .fill 03e0h-*,00h

             ;plugin creator(s)
             .db "BootGod"
             .fill 03f8h-*,00h

             ;PRG/CHR/WRAM min/max sizes supported
             .db 32 >> 4,128 >> 4
             .db 16 >> 3,32 >> 3
             .db 0,0

             ;plugin version and flags
             .db p_ver
             .db p_read

;plugin code start here
 

;determine H or V mirroring
             ldx #00h       
             lda #020h
             jsr wr_ppu
             lda #055h
             sta 2007h
             lda #0aah
             sta 2007h
             lda #024h
             jsr wr_ppu
             lda 2007h
             lda 2007h
             cmp #055h
             bne horz_mir
             lda 2007h
             cmp #0aah
             bne horz_mir
             lda #020h
             jsr wr_ppu
             lda #0aah
             sta 2007h
             lda #024h
             jsr wr_ppu
             lda 2007h
             lda 2007h
             cmp #0aah
             bne horz_mir
             inx		;vertical

horz_mir:    txa
             jsr send_byte	;send mirroring byte
             
;Get CRC of 1st PRG bank (32K at $8000)
             lda #00h
             jsr wr_bank
             sta address_lo
             lda #080h
             sta address_hi
             
             jsr init_crc
             ldy #0

chk_loop:    lda (address),y
             jsr do_crc
             iny
             bne chk_loop
             inc address_hi
             bne chk_loop
             jsr finish_crc

;Save 1st bank CRC for comparison
             ldx #3
ld_crc:      lda crc0,x
             sta temp_crc,x
             dex
             bpl ld_crc

;Get CRC of 2nd PRG bank
             lda #01h
             jsr wr_bank
             lda #0
             sta address_lo
             lda #080h
             sta address_hi
             
             jsr init_crc
             ldy #0

chk_loop2:   lda (address),y
             jsr do_crc
             iny
             bne chk_loop2
             inc address_hi
             bne chk_loop2
             jsr finish_crc

             ldy #1		;number of 32K PRG banks

;Compare 1st and 2nd bank CRCs
             ldx #3
chk_crc:     lda crc0,x             
             cmp temp_crc,x
             bne crc_fail
             dex
             bpl chk_crc

             jmp got_prg	;crcs matched, so is 32K


;Get CRC of 3rd PRG bank
crc_fail:    lda #02h
             jsr wr_bank
             lda #0
             sta address_lo
             lda #080h
             sta address_hi
             
             jsr init_crc
             ldy #0

chk_loop3:   lda (address),y
             jsr do_crc
             iny
             bne chk_loop3
             inc address_hi
             bne chk_loop3
             jsr finish_crc

             ldy #4

;Compare 1st and 3rd bank CRCs
             ldx #3
chk_crc2:    lda crc0,x             
             cmp temp_crc,x
             bne got_prg	;crcs didn't match, so is 128K
             dex
             bpl chk_crc2

             ldy #2		;crcs matched, so is 64K

;Send PRG header
got_prg:     tya
             lsr a
             pha
             lda #0
             ror a
             jsr send_byte
             pla
             jsr send_byte
             lda #01h
             jsr send_byte

;X will = # of 32K banks to send
             tya
             tax
             lda #0
             sta curbank

;Send current 32K bank
send_bank:   lda curbank
             jsr wr_bank
             inc curbank
             lda #0
             tay
             sta address_lo
             lda #080h
             sta address_hi

send_blk:    lda (address),y
             jsr send_byte
             iny
             bne send_blk
             inc address_hi
             bne send_blk 
             dex
             bne send_bank

             
;Get CRC of 1st CHR bank (8K at PPU $0000)
             lda #0
             jsr wr_bank
             lda #0
             jsr wr_ppu
             jsr init_crc
             lda 2007h

             lda #020h		;8K worth of data
             sta address_lo
             ldy #0

qck_loop:    lda 2007h
             jsr do_crc
             iny
             bne qck_loop
             dec address_lo
             bne qck_loop
             jsr finish_crc

;Save 1st bank CRC for comparison
             ldx #3
qld_crc:     lda crc0,x
             sta temp_crc,x
             dex
             bpl qld_crc
             
;Get CRC of 3rd CHR bank
             lda #08h
             jsr wr_bank
             lda #0
             jsr wr_ppu
             jsr init_crc  ;init crc
             lda 2007h

             lda #020h		;8K worth of data
             sta address_lo
             ldy #0

qck_loop2:   lda 2007h
             jsr do_crc
             iny
             bne qck_loop2
             dec address_lo
             bne qck_loop2
             jsr finish_crc
             
             ldy #080h		;# of 256-byte pages (80h = 32K)

;Compare 1st and 3rd bank CRCs
             ldx #3
qchk_crc:    lda crc0,x             
             cmp temp_crc,x
             bne got_chr
             dex
             bpl qchk_crc

             ldy #040h		;40h = 16K

;Send CHR header
got_chr:     tya
             jsr send_byte
             lda #0
             jsr send_byte
             lda #02h
             jsr send_byte   ;send our header

;X will be # of 8K banks to send
             tya
             lsr a
             lsr a
             lsr a
             lsr a
             lsr a
             tax
             lda #0
             sta curbank

;Send current 8K CHR bank
send_cbank:  lda curbank
             asl a
             asl a
             jsr wr_bank
             inc curbank
             lda #0
             jsr wr_ppu
             lda 2007h
             lda #020h
             sta address_hi
             ldy #0

send_cblk:   lda 2007h
             jsr send_byte
             iny
             bne send_cblk
             dec address_hi
             bne send_cblk
             dex
             bne send_cbank
             
;Send termination header
             lda #000h
             jsr send_byte
             lda #000h
             jsr send_byte
             lda #000h
             jsr send_byte

             rts

wr_bank:     sta 7000h
             rts

             .fill 0800h-*,0ffh   ;fill rest to get 1K of data

             .end
